@@ -78,6 +78,8 @@ class priorityLogger(object):
         self.mjd = list()
         self.field_pk = list()
         self.field_id = list()
+        self.ra = list()
+        self.dec = list()
         self.cadence = list()
         self.cadencePriority = list()
         self.priority = list()
@@ -85,15 +87,19 @@ class priorityLogger(object):
         self.model = [('mjd', np.float32),
                       ('field_pk', np.int32),
                       ('field_id', np.int32),
+                      ('ra', np.float64),
+                      ('dec', np.float64),
                       ('cadence', np.dtype('a20')),
                       ('cadencePriority', np.float32),
                       ('priority', np.float32)]
 
     def add(self, mjd=None, field_pk=None, field_id=None, cadence=None,
-            cadencePriority=None, priority=None):
+            cadencePriority=None, priority=None, ra=None, dec=None):
         self.mjd.append(mjd)
         self.field_pk.append(field_pk)
         self.field_id.append(field_id)
+        self.ra.append(ra)
+        self.dec.append(dec)
         self.cadence.append(cadence)
         self.cadencePriority.append(cadencePriority)
         self.priority.append(priority)
@@ -102,6 +108,8 @@ class priorityLogger(object):
         self.mjd = list()
         self.field_pk = list()
         self.field_id = list()
+        self.ra = list()
+        self.dec = list()
         self.cadence = list()
         self.cadencePriority = list()
         self.priority = list()
@@ -121,6 +129,8 @@ class priorityLogger(object):
         output["mjd"] = np.array(self.mjd)
         output["field_pk"] = np.array(self.field_pk)
         output["field_id"] = np.array(self.field_id)
+        output["ra"] = np.array(self.ra)
+        output["dec"] = np.array(self.dec)
         output["cadence"] = np.array(self.cadence)
         output["cadencePriority"] = np.array(self.cadencePriority)
         output["priority"] = np.array(self.priority)
@@ -988,8 +998,10 @@ class Scheduler(Master):
                                      dec=self.fields.deccen)
         skybrightness = self.skybrightness(mjd)
         # valid cadence checks against "none" cadence issue
-        observable = (alt > 0.) & self.fields.validCadence & self.fields.notDone
+        observable = (alt > 30.) & self.fields.validCadence & self.fields.notDone
         nexp = np.ones(len(observable), dtype=int)
+        exp_epochs = np.zeros(len(observable), dtype=np.int32)
+        epoch_idxs = np.zeros(len(observable), dtype=np.int32)
         delta_priority = np.zeros(len(observable), dtype=np.float64)
 
         deltav = self.deltaV_sky_pos(mjd, self.fields.racen, self.fields.deccen)
@@ -1025,12 +1037,12 @@ class Scheduler(Master):
 
         # print(f"{float(mjd):.3f} {float(next_change):.3f} {float(next_brightness):.2f} {nexp_change}", maxExp)
 
-        over = list()
+        # over = list()
 
         # indxs = np.where(self.fields.nextmjd > mjd)[0]
         # observable[indxs] = False
         indxs = np.where(observable)[0]
-        # print(f"attempting {mjd:.2f} with {len(indxs)} fields")
+        # print(f"attempting {float(mjd):.2f} with {len(indxs)} fields")
         for indx in indxs:
             # if(observable[indx]):
             if int(self.fields.pk[indx]) in ignore:
@@ -1041,54 +1053,39 @@ class Scheduler(Master):
                 continue
             cadence = self.cadencelist.cadences[self.fields.cadence[indx]]
 
+            if int(self.fields.epoch_idx[indx]) >= cadence.nepochs and self.fields.flag[indx] != 1:
+                observable[indx] = False
+                continue
+
             mjd_past = self.fields.hist[self.fields.pk[indx]]
             # epoch_idx is the *index* of the *next* epoch
-            # for 0 indexed arrays, this equivalent to
-            # "how many epochs have I done previously"
-            tol = cadence.max_length[int(self.fields.epoch_idx[indx])]
-            if tol < 0.1:
-                tol = 0.1
-            elif tol > 14:
-                tol = 14
-            epoch_idx, begin_last_epoch = epochs_completed(mjd_past, tolerance=tol)
-            # how many exp/"designs" since start of last epoch?
-            exp_epoch = np.sum(np.greater_equal(mjd_past, begin_last_epoch))
+            expCount = [np.sum(cadence.nexp[:i+1]) for i in range(len(cadence.nexp))]
+            epoch_idx = np.where(np.array(expCount) > len(mjd_past))[0][0]
+
+            if epoch_idx >= cadence.nepochs and self.fields.flag[indx] != 1:
+                observable[indx] = False
+                continue
+
+            if epoch_idx > 0:
+                exp_epoch = expCount[epoch_idx-1] - len(mjd_past)
+            else:
+                exp_epoch = len(mjd_past)
+
+            nexp[indx] = cadence.nexp[epoch_idx] - exp_epoch
+            partial_epoch = exp_epoch > 0
+
+            exp_epochs[indx] = exp_epoch
+            epoch_idxs[indx] = epoch_idx
+
             if len(mjd_past):
                 mjd_prev = mjd_past[-1]
             else:
                 mjd_prev = 0
-            if len(mjd_past) > np.sum(cadence.nexp):
-                over.append(int(self.fields.pk[indx]))
-            actually_done = False
-            # because dark-2xN can have max length 7 but delta min 0.5
-            while exp_epoch > cadence.nexp[epoch_idx]:
-                exp_epoch -= cadence.nexp[epoch_idx]
-                epoch_idx += 1
-                if epoch_idx == cadence.nepochs:
-                    actually_done = True
-                    break
-            if actually_done:
-                observable[indx] = False
-                continue
-            if exp_epoch < cadence.nexp[epoch_idx] and exp_epoch != 0:
-                nexp[indx] = cadence.nexp[epoch_idx] - exp_epoch
-                epoch_idx -= 1
-                partial_epoch = True
-            if epoch_idx >= cadence.nepochs and not self.fields.flag[indx] == 1:
-                # print("DONE ", epoch_idx, cadence.nepochs, int(self.fields.pk[indx]))
-                observable[indx] = False
-                continue
-            else:
-                nexp[indx] = cadence.nexp[epoch_idx]
-                partial_epoch = False
+
             if nexp[indx] > nexp_change and self.fields.flag[indx] != 1:
-                # print(f"{float(mjd):.3f} {int(self.fields.pk[indx])} c_nexp {cadence.nexp[epoch_idx]} NEXP {nexp_change} MAXEXP {maxExp}")
                 # change between bright/dark, field doesn't fit
                 observable[indx] = False
                 continue
-            # if int(self.fields.pk[indx]) in over:
-            #     print("!?", epoch_idx >= cadence.nepochs, not self.fields.flag[indx] == 1)
-            #     print(int(self.fields.pk[indx]), len(mjd_past), cadence.nexp, exp_epoch, cadence.nepochs, epoch_idx)
             observable[indx], delta_priority[indx] =\
                 cadence.evaluate_next(epoch_idx=epoch_idx,
                                       partial_epoch=partial_epoch,
@@ -1098,12 +1095,6 @@ class Scheduler(Master):
                                       moon_dist=moon_dist[indx],
                                       deltaV=deltav[indx],
                                       airmass=airmass[indx])
-            # if nexp[indx] > 6 and observable[indx]:
-            #     delta_priority[indx] += 1e6
-            # delta_priority[indx] += 2.5**nexp[indx]
-
-            # if not observable[indx]:
-            #     print(f"{float(mjd):.3f} {int(self.fields.pk[indx])} {cadence.name} MAXEXP {maxExp}")
 
             delta_priority[indx] += nExpPrioritize(nexp[indx],
                                                    base=self.nExpPriBase,
@@ -1123,21 +1114,11 @@ class Scheduler(Master):
                 # so override cadence eligibility and bump priority
                 observable[indx] = True
                 delta_priority[indx] += 1e6
-            # if "dark_2x2" in cadence.name and observable[indx] and mjd_prev != 0:
-            #     self.check = True
-            #     # print(f"F {float(mjd):.3f} {int(self.fields.pk[indx])} {delta_priority[indx]}")
-            #     self.recent.append(f"R {int(self.fields.pk[indx])} {epoch_idx} prev {len(mjd_past)} epoch {epoch_idx} tol {tol} partial {partial_epoch}")
-            #     self.recent_ids.append(int(self.fields.pk[indx]))
-                # print(nexp[indx], type(nexp[indx]))
-            # elif "x8" in cadence.name and not observable[indx]:
-            #     if skybrightness < 0.36 and moon_dist[indx] < 30 and deltav[indx] < 2 and airmass[indx] < 1.5:
-            #         print(f"F {epoch_idx} {float(mjd):.3f} {int(self.fields.pk[indx])} {mjd-mjd_prev}")
-            #         print(f"sky {skybrightness} moon {moon_dist[indx]} deltav {deltav[indx]} AM {airmass[indx]:}")
-            #         print(nexp[indx], type(nexp[indx]), maxExp)
-        # print(f"{float(mjd):.3f} {float(next_change):.3f} {float(next_brightness):.2f} {nexp_change}", maxExp)
+
         iobservable = np.where(observable)[0]
 
-        return iobservable, nexp[iobservable], delta_priority[iobservable]
+        return iobservable, nexp[iobservable], delta_priority[iobservable],\
+            exp_epochs[iobservable], epoch_idxs[iobservable]
 
     def prioritize(self, mjd=None, iobservable=None, nexp=None,
                    delta_priority=None):
@@ -1189,7 +1170,8 @@ class Scheduler(Master):
         assert len(lstDiffs) == len(iobservable), "lst weight going poorly"
         # assert 0 not in delta_remaining, "some delta remaining not set properly!"
 
-        ha = self.ralst2ha(ra=self.fields.racen[iobservable], lst=lst)
+        ra = self.fields.racen[iobservable]
+        ha = self.ralst2ha(ra=ra, lst=lst)
         dec = self.fields.deccen[iobservable]
 
         # gaussian weight, mean already 0, use 1 hr  std
@@ -1203,10 +1185,10 @@ class Scheduler(Master):
         field_id = self.fields.field_id[iobservable]
         cadence = [self.fields.cadence[i] for i in iobservable]
 
-        for k, i, c, d, p in zip(pk, field_id, cadence, delta_priority, priority):
+        for k, i, c, d, p, r, de in zip(pk, field_id, cadence, delta_priority, priority, ra, dec):
             self.priorityLogger.add(mjd=mjd, field_pk=k, field_id=i,
                                     cadence=c, cadencePriority=d,
-                                    priority=p)
+                                    priority=p, ra=r, dec=de)
 
         return priority
 
@@ -1247,7 +1229,7 @@ class Scheduler(Master):
 
         return(pick_fieldid, pick_exp)
 
-    def designsNext(self, field_pk):
+    def designsNext(self, field_pk, pick_exp_epoch, pick_epoch_idx):
         """Figure out next designs on a field, i.e. which exposure are we on?
 
         Parameters:
@@ -1267,17 +1249,19 @@ class Scheduler(Master):
         fieldidx = self.fields.getidx(field_pk)
         mjd_past = self.fields.hist[field_pk]
         cadence = self.cadencelist.cadences[self.fields.cadence[fieldidx]]
-        epoch_idx, mjd_prev = epochs_completed(mjd_past, tolerance=240)
-        nexp_next = cadence.nexp[epoch_idx]
+        # epoch_idx, mjd_prev = epochs_completed(mjd_past, tolerance=tol)
+        nexp_next = cadence.nexp[pick_epoch_idx]
         # how many completed designs since start of epoch
         # this assumes designs are marked incomplete when epoch_max_length
         # is hit
-        exp_in_epoch = np.sum(np.greater(mjd_past, mjd_prev))
+        # exp_in_epoch = np.sum(np.greater(mjd_past, mjd_prev))
 
-        exp_to_do = nexp_next - exp_in_epoch
+        exp_to_do = nexp_next - pick_exp_epoch
 
         designs = list(len(mjd_past) + np.arange(exp_to_do))
 
+        print(self.fields.field_id[fieldidx], len(mjd_past), designs)
+
         return designs
 
     def nextfield(self, mjd=None, maxExp=None, returnAll=False, live=False,
@@ -1319,13 +1303,13 @@ class Scheduler(Master):
         self.recent = list()
         self.recent_ids = list()
 
-        iobservable, nexp, delta_priority = self.observable(mjd=mjd, maxExp=maxExp,
-                                                            ignore=ignore)
-        if(len(iobservable) == 0) and live:
-            # in a sim we'll take the dead time, should write something to track this better
-            # print("Nothing observable")
-            iobservable, nexp, delta_priority = self.observable(mjd=mjd, maxExp=maxExp,
-                                                                ignore=ignore)
+        iobservable, nexp, delta_priority, exp_epoch, epoch_idx\
+            = self.observable(mjd=mjd, maxExp=maxExp, ignore=ignore)
+        # if(len(iobservable) == 0) and live:
+        #     # in a sim we'll take the dead time, should write something to track this better
+        #     # print("Nothing observable")
+        #     iobservable, nexp, delta_priority, exp_epoch, epoch_idx\
+        #         = self.observable(mjd=mjd, maxExp=maxExp, ignore=ignore)
         if len(iobservable) == 0:
             # print("!! nothing to observe; {} exp left in the night".format(maxExp))
             if returnAll:
@@ -1363,16 +1347,21 @@ class Scheduler(Master):
 
         observable_fieldid = self.fields.pk[iobservable]
 
-        field_pk, next_exp = self.pick(priority=priority,
-                                       fieldid=observable_fieldid,
-                                       nexp=nexp)
+        # field_pk, next_exp = self.pick(priority=priority,
+        #                                fieldid=observable_fieldid,
+        #                                nexp=nexp)
+        ipick = np.argmax(priority)
+        field_pk = observable_fieldid[ipick]
+        next_exp = nexp[ipick]
+        pick_exp_epoch = exp_epoch[ipick]
+        pick_epoch_idx = epoch_idx[ipick]
 
         if not live:
             # just a sim return number of designs
             return(field_pk, next_exp)
 
         # its live, return list of exp indices
-        designs = self.designsNext(field_pk)
+        designs = self.designsNext(field_pk, pick_exp_epoch, pick_epoch_idx)
 
         return field_pk, designs
 
